//--------------------------- BASIC PROPERTIES ------------------------------

#include "include\\Warnings.fxh"

float4x4 World;
float4x4 View;
float4x4 Projection;
float4x4 WorldInverseTranspose;

static const int MAX_MATRICES = 35;
float4x4 WorldMatrixArray[MAX_MATRICES] : WORLDMATRIXARRAY;
int numBoneInfluences = 2;

bool Skinning;
bool SelfShadows;

float3 DiffuseLightDirection;
float4 DiffuseColor = float4(1, 1, 1, 1);
float DiffuseIntensity = 1.0;
float4 LineColor = float4(0, 0, 0, 1);
float LineThickness;

texture2D Texture;
 
// The texture sampler, which will get the texture color
SamplerState textureSampler = sampler_state
{
    Texture = (Texture);
    Filter = COMPARISON_MIN_MAG_LINEAR_MIP_POINT;
    AddressU = Clamp;
    AddressV = Clamp;
};
 
//--------------------------- DATA STRUCTURES ------------------------------
struct VS_INPUT
{
    float4 Position : POSITION0;            // The position of the vertex
    float3 Normal : NORMAL0;                // The vertex's normal
    float2 TextureCoordinate : TEXCOORD0;    // The texture coordinate of the vertex
	float4 BlendWeights : BLENDWEIGHT0;		//
    float4 BlendIndices : BLENDINDICES0;	//
};
 
struct VS_OUTPUT
{
    float4 Position : POSITION0;
    float2 TextureCoordinate : TEXCOORD0;
    float3 Normal : TEXCOORD1;
};

//--------------------------- SHADERS ------------------------------

// The vertex shader that does cel shading.
// It really only does the basic transformation of the vertex location,
// and normal, and copies the texture coordinate over.
VS_OUTPUT main(VS_INPUT input)
{
    VS_OUTPUT output; 

	float4 p = float4(0, 0, 0, 0);
	float3 norm = float3(0, 0, 0);
	int n = numBoneInfluences-1;
	input.Normal = normalize(input.Normal);

	if(Skinning)
	{
		// cast the vectors to arrays for use in the for loop below
		float BlendWeightsArray[4] = (float[4])input.BlendWeights;

		if(1)
		{			
			float lastWeight = 0.0f;
    
			//Blend vertex position & normal
			for(int i = 0; i < n; ++i)
			{
				lastWeight += BlendWeightsArray[i];
				p += BlendWeightsArray[i] * mul(input.Position, WorldMatrixArray[input.BlendIndices[i]]);
				norm += BlendWeightsArray[i] * mul(input.Normal, WorldMatrixArray[input.BlendIndices[i]]);
			}
			lastWeight = 1.0f - lastWeight;
    
			p += lastWeight * mul(input.Position, WorldMatrixArray[input.BlendIndices[n]]);
			norm += lastWeight * mul(input.Normal, WorldMatrixArray[input.BlendIndices[n]]);
			p.w = 1.0f;
		}
		else
		{ 
			// calculate the pos/normal
			for(int i = 0; i < n; ++i)
			{
				p += BlendWeightsArray[i] * mul(input.Position, WorldMatrixArray[input.BlendIndices[i]]);
				norm += BlendWeightsArray[i] * mul(input.Normal, WorldMatrixArray[input.BlendIndices[i]]);
			}
		}
	}
	else
	{
		p = mul(input.Position, World);
		norm = mul(input.Normal, World);
	}

    // Transform the position
    output.Position = mul(mul(p, View), Projection);
 
    // Transform the normal
    output.Normal = normalize(mul(norm, WorldInverseTranspose));
 
    // Copy over the texture coordinate
    output.TextureCoordinate = input.TextureCoordinate;
 
    return output;
}